function [m,v] = sim_state(m_bar,v_bar,kap_m,kap_v,nu_m,nu_v,T,freq,prec,full)
%----------------------------------------------------------------------
% PURPOSE: Simulate series of square root process (for mu and v)
%======================================================================
%  NOTES:
%  Modified from the code written by Wacther (2013, Journal of Finance)
%======================================================================

%======================================================================
%  INITIALIZATIONS
%======================================================================
m    = zeros(T*freq*prec+1,1);
v    = zeros(T*freq*prec+1,1);
uu_m = randn(T*freq*prec,1)/sqrt(freq*prec);  %shocks to mu
uu_v = randn(T*freq*prec,1)/sqrt(freq*prec);  %shocks to v

if m_bar(1)<0 && v_bar(2)<0
  error('average state variables negative')
end

m(1)    = m_bar;  
v(1)    = v_bar;  
m_prev  = m(1);
v_prev  = v(1);

for t= 1:T*freq*prec
    if full
        %use the previous values of v(t) even if they are truncated.
        v_new = v(t) - kap_v/(freq*prec).*(v(t)-v_bar) + nu_v.*sqrt(v(t)).*uu_v(t);
    else
        %"partial" truncation -- keep the previous v_new even if negative for use in the drift term.
        v_new = v(t) - kap_v/(freq*prec).*(v_prev-v_bar) + nu_v.*sqrt(v(t)).*uu_v(t);
    end
    if v_new(1)<0
        v(t+1) = 0;
    else
        v(t+1) = v_new;
    end
    v_prev   = v_new;
end

for t= 1:T*freq*prec
    if full
        %use the previous values of mu(t) even if they are truncated.
        m_new = m(t) - kap_m/(freq*prec).*(m(t)-m_bar) + nu_m.*sqrt(v(t)).*uu_m(t);
    else
        %"partial" truncation -- keep the previous mu_new even if negative for use in the drift term.
        m_new = m(t) - kap_m/(freq*prec).*(m_prev-m_bar) + nu_m.*sqrt(v(t)).*uu_m(t);
    end
    m(t+1) = m_new;
    m_prev = m_new;
end
  
% End-of-file
