%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

MKTX = Robs_risk'-Robs_bill';
MKT   = Robs_bill';
RB   = Robs_bill';
RF   = Robs_free'; % by assumption riskfree

% collect data in matrix
rawdata = [Robs_free' RF MKTX gobs'];
temp=~any(isnan([Robs_free' RF MKTX gobs'])')';
rawdata = rawdata(temp,:);
MKTX = MKTX(temp,:);
MKT  = MKT(temp,:);
RB   = RB(temp,:);
RF   = RF(temp,:);

% annualized in percentage points
EP    = mean(MKTX/freq);           % mean of equity excess returns over the bill rate (equity premium)
EPSD  = sqrt(var(MKTX/freq));      % sd of equity excess returns over the bill rate (equity premium)
MRF   = mean(RF-1)/freq;           % mean of bill rate
MRFSD = sqrt(var((RF-1)/freq));    % sd of bill rate
MR    = mean((MKTX+RF)-1)/freq;          % mean of equity rate of return
MRSD  = sqrt(var(((MKTX+RF)-1)/freq));   % sd of equity rate of return

nz = 0;                     % Number of lags used as instruments
T = size(rawdata,1)-nz;     % sample size    
neq = 1;                    % no of equations

% get the consumption ratio from log growth rates
cg = exp(rawdata(1+nz:T+nz,4));
R = [rawdata(1+nz:T+nz,2) rawdata(1+nz:T+nz,3)];   % vector of returns [MKT MKTX]

% C-CAPM structural parameters
b_name = strvcat('beta','gamma');                                           %#ok<DSTRVCT> % variable names
% initial values
b = [1;10];                                                                 

% select algorithm
algo='Levenberg-Marquardt';
% options= optimset('Display','off','MaxIter', 200000, 'MaxFunEvals', 200000,'TolFun', sqrt(eps),'Algorithm',algo, 'Jacobian', 'off', 'UseParallel','always');
options= optimset('Display','off','MaxIter', 5000, 'MaxFunEvals', 5000,'TolX',1e-12,'TolFun', sqrt(eps),'Algorithm',algo, 'Jacobian', 'off');

%% Estimation of structural parameters
if (solution == 1)
    % MEF and GMM with two conditional moment restrictions (riskless, risky asset)
    X = [cg MKT MKTX];
    Z = ones(T,1);                % vector of instruments
    y = [ones(T,1) ones(T,1)];   % matrix containing the population mean of moment conditions
    
    params = [rho,gamma,mu_bar,sigma_bar,nu_bar,lambda,kappa,q,freq];
    
    [b_2_gmm,fval_2_gmm,exitflag_2_gmm] = fminsearch(@gmm_2_func,b(1:2),options,y,X,Z);  % Call minimizer
    [b_2_cond_gmm,fval_2_cond_gmm,exitflag_2_cond_gmm] = fminsearch(@gmm_2_cond_func,b,options,y,X,Z,params);  % Call minimizer
    [b_2_cond2_gmm,fval_2_cond2_gmm,exitflag_2_cond2_gmm] = fminsearch(@gmm_2_cond2_func,b,options,y,X,Z,params);  % Call minimizer
    
    bopt=b_2_gmm;
    fval=fval_2_gmm;
    exitflag=exitflag_2_gmm;
    bopt_cond=b_2_cond_gmm;
    bopt_cond2=b_2_cond2_gmm;
    fval_cond=fval_2_cond_gmm;
    exitflag_cond=exitflag_2_cond_gmm;
    
    gamma_hat = gamma-(exp(-gamma*nu_bar)*(exp(nu_bar)-1)*lambda+(1-exp(kappa))*q*exp(-gamma*nu_bar)*lambda)/sigma_bar^2;
    beta_hat = exp(-rho*freq)*exp(-(gamma-gamma_hat)*(mu_bar-1/2*sigma_bar^2)*freq+0.5*(gamma^2-gamma_hat^2)*sigma_bar^2*freq-((1-(1-exp(kappa))*q)*exp(-gamma*nu_bar)-1)*lambda*freq);

elseif solution == 3
    X = [cg MKT MKTX];    
    Z = ones(T,1);                % vector of instruments
    y = [ones(T,1) ones(T,1)];   % matrix containing the population mean of moment conditions

    b = [1;10];
    
    [b_2_gmm,fval_3_gmm,exitflag_3_gmm] = fminsearch(@gmm_2_func3,b(1:2),options,y,X,Z);  % Call minimizer
    [b_2_cond_gmm,fval_3_cond_gmm,exitflag_3_cond_gmm] = fminsearch(@gmm_2_cond_func3,b,options,y,X,Z);  % Call minimizer
    [b_2_cond2_gmm,fval_3_cond2_gmm,exitflag_3_cond2_gmm] = fminsearch(@gmm_2_cond2_func3,b,options,y,X,Z);  % Call minimizer
 
    bopt=b_2_gmm;
    fval=fval_3_gmm;
    exitflag=exitflag_3_gmm;
    bopt_cond=b_2_cond_gmm;
    bopt_cond2=b_2_cond2_gmm;
   
elseif solution == 4
    X = [cg MKT MKTX];    
    Z = ones(T,1);                % vector of instruments
    y = [ones(T,1) zeros(T,1)];   % matrix containing the population mean of moment conditions
    
    b = [1;10];
    
    [b_2_gmm,fval_3_gmm,exitflag_3_gmm] = fminsearch(@gmm_2_func4,b(1:2),options,y,X,Z);  % Call minimizer
    
    bopt=b_2_gmm;
    fval=fval_3_gmm;
    exitflag=exitflag_3_gmm;
end

sdfpar=bopt;
if solution ~= 4
    sdfpar_cond=bopt_cond;
    sdfpar_cond2=bopt_cond2;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Euler equation errors
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sdf_fitted=sdfpar(1,:)*cg.^(-sdfpar(2,:));  % fitted SDF
if solution~= 4
    sdf_fitted_cond=sdfpar_cond(1,:)*cg.^(-sdfpar_cond(2,:));  % fitted SDF
    sdf_fitted_cond2=exp(-rho*freq)*cg.^(-sdfpar_cond2(2,:));  % fitted SDF
end

% estimated Euler equation errors bill (euler1) and excess returns (euler2)
% euler1=repmat(sdf_fitted,1,1).*RB - ones(T,1);
% euler2=repmat(sdf_fitted,1,1).*R(:,2) - zeros(T,1);
euler1=repmat(sdf_fitted,1,1).*MKT - ones(T,1);
euler2=repmat(sdf_fitted,1,1).*MKTX;

if solution ~= 4
    euler1_cond=repmat(sdf_fitted_cond,1,1).*RB - ones(T,1);
    euler2_cond=repmat(sdf_fitted_cond,1,1).*MKT - ones(T,1);
%     euler1_cond2=repmat(sdf_fitted_cond2,1,1).*MKT - ones(T,1);
%     euler2_cond2=repmat(sdf_fitted_cond2,1,1).*MKTX - ones(T,1)*sdfpar_cond2(1);
end

if solution == 4
    euler3_correct_alternative=mean(Pd_qoq(2:end,loop)'./Pd_qoq(1:end-1,loop)'-mean(mean(Pd_qoq(2:end,:)'./Pd_qoq(1:end-1,:)',1)),2);
    euler1_correct_alternative=mean(rbond_qoq(:,loop)'-mean(mean(rbond_qoq,1)),2);
    euler2_correct_alternative=mean(rc_qoq(:,loop)'-mean(mean(rc_qoq,1)),2);

    % 'correct' SDF of the wrongly specified C-CAPM SDF
    SDFobs = exp(-freq*rho)*cg.^(-gamma);
    euler2_correct = repmat(SDFobs,1,1).*MKTX;
    euler1_correct = repmat(SDFobs,1,1).*MKT - ones(T,1);
else
    euler2_correct=repmat(SDFobs',1,1).*R(:,2) - zeros(T,1);
    euler1_correct=repmat(SDFobs',1,1).*MKT - ones(T,1);
end

e1=mean(euler1);
e2=mean(euler2);
e1_correct=mean(euler1_correct);
e2_correct=mean(euler2_correct);
if solution ==4
    e1_correct_alternative=mean(euler1_correct_alternative);
    e2_correct_alternative=mean(euler2_correct_alternative);
    e3_correct_alternative=mean(euler3_correct_alternative);
end

% square root of the average squared EE Error
rmse=sqrt(0.5*(e1^2 + e2^2));                                              % RMSE
if solution == 4 
    rmse_correct=sqrt(1/3*(e1_correct_alternative^2 + e2_correct_alternative^2+ e3_correct_alternative^2));
%     rmse_correct=sqrt(0.5*(e1_correct^2 + e2_correct^2));
    rmsr=sqrt(0.5*((mean(X(:,3)))^2 + (mean(X(:,2)-1))^2));
    
    
else
    rmse_correct=sqrt(0.5*(e1_correct^2 + e2_correct^2));
    rmsr=sqrt(0.5*((mean(X(:,2)-X(:,3)) -1)^2 + (mean(X(:,2)-1))^2));
end

EPF = NaN;
RP = NaN;
e2_cond=0;
e1_cond=0;
e1_cond2=0;
e2_cond2=0;
sdfpar_cond=sdfpar;
sdfpar_cond2=sdfpar;
rmse_cond= 0;
rmse_cond2= 0;
%     rmse_correct = 0;
Nobs=0;
N2obs=0;
results_mat(loop,:) = [sdfpar' rmse/freq*100 rmse_correct/freq*100 EP*100 MR*100 MRF*100 EPF/freq*100 RP*100 mean(log(cg))/freq*100 EPSD*100 MRSD*100 MRFSD*100 sqrt(var(log(cg)/freq))*100 e1/freq*100 e1_correct/freq*100 e2/freq*100 e2_correct/freq*100 max(N2obs) max(Nobs) exitflag NaN NaN NaN sdfpar_cond' rmse_cond/freq*100 e1_cond/freq*100 e2_cond/freq*100 rmsr/freq*100 rmse/rmsr sdfpar_cond2' rmse_cond2/freq*100 e1_cond2/freq*100 e2_cond2/freq*100];

%% Auxiliary functions
function [f,b]=gmm_2_func(b,y,X,Z)
[T, neq] = size(y);
k = size(Z,2);
north = neq*k;
RF = X(:,2)-X(:,3);
MKT = X(:,2);
R = [RF MKT];
cg = X(:,1);
e = repmat(b(1)*cg.^(-b(2)),1,neq).*R - y;
m = reshape(Z'*e/T,north,1);
W=eye(k);       % use identity matrix
f=m'*W*m;       % setup objective function
end

function [f,b]=gmm_2_cond_func(b,y,X,Z,params)
[T, neq] = size(y);
k = size(Z,2);
north = neq*k;
R = [X(:,2)-X(:,3) X(:,2)];
cg = X(:,1);
imrs = b(1)*cg.^(-b(2));
nu_bar=params(5);
lambda=params(6);
kappa=params(7);
q=params(8);
freq = params(9);
gamma = b(2);
e = repmat(imrs,1,neq).*R - y*[exp((1-exp(-gamma*nu_bar))*lambda*freq+exp(-gamma*nu_bar)*(1-exp(kappa))*q*lambda*freq),0;0,exp(-(exp((1-gamma)*nu_bar)-1)*lambda*freq)];
m = reshape(Z'*e/T,north,1);
Winit=eye(k);       % use identity matrix
f=m'*Winit*m;       % setup objective function
end

function [f,b]=gmm_2_cond2_func(b,y,X,Z,params)
[T, neq] = size(y);
k = size(Z,2);
north = neq*k;
R = [X(:,2) X(:,3)];
cg = X(:,1);
rho=params(1);
freq = params(9);
imrs = exp(-rho*freq)*cg.^(-b(2));
e = repmat(imrs,1,neq).*R - y*[1,0;0,b(1)];
m = reshape(Z'*e/T,north,1);
Winit=eye(k);       % use identity matrix
f=m'*Winit*m;       % setup objective function
end

function [f,b]=gmm_2_func3(b,y,X,Z)
[T, neq] = size(y);
k = size(Z,2);
north = neq*k;
R = [X(:,2)-X(:,3) X(:,2)];
cg = X(:,1);
imrs = b(1)*cg.^(-b(2));
e = repmat(imrs,1,neq).*R - y;
m = reshape(Z'*e/T,north,1);
Winit=eye(k);       % use identity matrix
f=m'*Winit*m;       % setup objective function
end

function [f,b]=gmm_2_cond_func3(b,y,X,Z)
[T, neq] = size(y);
k = size(Z,2);
north = neq*k;
R = [X(:,2)-X(:,3) X(:,2)];
cg = X(:,1);
nu=-0.55;
freq = 1/4;
imrs = b(1)*cg.^(-b(2));
lambda=0.017;
alpha=0.6;
gamma=b(2);
e = repmat(imrs,1,neq).*R - y*[exp((1-exp(-alpha*gamma*nu))*lambda*freq),0;0,1];
m = reshape(Z'*e/T,north,1);
Winit=eye(k);       % use identity matrix
f=m'*Winit*m;       % setup objective function
end

function [f,b]=gmm_2_cond2_func3(b,y,X,Z)
[T, neq] = size(y);
k = size(Z,2);
north = neq*k;
R = [X(:,2)-X(:,3) X(:,3)];
cg = X(:,1);
freq = 1/4;
mu_bar     = 0.01;       % drift TFP process
sigma_bar  = .01;      % volatility TFP process
nu_bar     = -.01;         % jump size TFP process
lambda_bar = 0*0.2;         % arrival rate TFP jumps
nu         = -.55;       % jump size physical capital
lambda     = .017;      % arrival rate jumps in physical capital
sigma      = 0.005;      % volatility physical capital (determines variation in the equity premium)
gamma = 4;
alpha = .6;
delta = .025;
rho = ((exp(-gamma*nu_bar)-1)*lambda_bar+(exp((1-alpha*gamma)*nu)-1)*lambda-gamma*mu_bar+.5*(gamma*(1+gamma)*sigma_bar^2-alpha*gamma*(1-alpha*gamma)*sigma^2)-(1-alpha*gamma)*delta);
imrs = exp(-rho*freq)*cg.^(-b(2));
e = repmat(imrs,1,neq).*R - y*[1-b(1),0;0,b(1)];
m = reshape(Z'*e/T,north,1);
Winit=eye(k);       % use identity matrix
f=m'*Winit*m;       % setup objective function
end

function [f,b]=gmm_2_func4(b,y,X,Z)
[T, neq] = size(y);
k = size(Z,2);
north = neq*k;
R = [X(:,2) X(:,3)];
cg = X(:,1);
imrs = b(1)*cg.^(-b(2));
e = repmat(imrs,1,neq).*R - y;
m = reshape(Z'*e/T,north,1);
Winit=eye(k);       % use identity matrix
f=m'*Winit*m;       % setup objective function
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% End of file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
