%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sampling paths
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Realization of stochastic processes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if loop > 999 sample = num2str(loop); end
if loop < 1000 sample = ['0',num2str(loop)]; end
if loop < 100 sample = ['00',num2str(loop)]; end
if loop < 10 sample = ['000',num2str(loop)]; end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Write sample paths to files or read sample paths from files
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (stochastics==1) 
    main_stochastics;
    
    outr_xnorm=[outr_xnorm xnorm'];
    outr_xnorm2=[outr_xnorm2 xnorm2'];
    outr_xnorm3=[outr_xnorm3 xnorm3'];
    outr_xpois=[outr_xpois xpois'];
    outr_xpois2=[outr_xpois2 xpois2'];
    % auxiliary variables
    [i_x,j_x]=size(xjump);
    outr_xjump=[outr_xjump [xjump(:,1);j_x]];
    [i_x2,j_x2]=size(xjump2);
    outr_xjump2=[outr_xjump2 [xjump2(:,1);j_x2]];
    [i_d1,j_d1]=size(xdefs);
    outr_xdefs=[outr_xdefs [xdefs(:,1);j_d1]];
    [i_b1,j_b1]=size(xbins);
    outr_xbins=[outr_xbins [xbins(:,1);j_b1]];
else
    % load variables from Matlab file
    load(['Data/Draw_' startdate '/Samples/'  'RES_SAMP.mat']);
    xnorm=outr_xnorm(:,loop);
    xnorm2=outr_xnorm2(:,loop);
    xnorm3=outr_xnorm3(:,loop);
    xpois=outr_xpois(:,loop);
    xpois2=outr_xpois2(:,loop);
    % auxiliary variables
    qxjump=outr_xjump(:,loop);    
    xjump=repmat(qxjump(1),qxjump(2),1);
    qxjump2=outr_xjump2(:,loop);       
    xjump2=repmat(qxjump2(1),qxjump2(2),1);
    qxdefs=outr_xdefs(:,loop);
    xdefs=repmat(qxdefs(1),qxdefs(2),1);
    qxbins=outr_xbins(:,loop); 
    xbins=repmat(qxbins(1),qxbins(2),1);
end

% fix that rows and cols may differ with sample size
if (size(xnorm,1)>1) 
    xjump=xjump'; 
    xjump2=xjump2'; 
    xpois=xpois'; 
    xpois2=xpois2'; 
    xnorm=xnorm'; 
    xnorm2=xnorm2'; 
    xnorm3=xnorm3'; 
    xdefs=xdefs'; 
    xbins=xbins'; 
end
% impose no-disaster sample if required
if (quiet ==1)    
    if (solution == 1) 
        xpois(1:end)=0; 
    else
        xpois2(1:end)=0;
    end
end

% create initial values
Asim_init = 1;
Bsim_init = 0;
Zsim_init = 0;
Z2sim_init = 0;
Nsim_init = 0;
N2sim_init = 0;

% simulated process
Bsim = cumsum(xnorm)*sqrt(prec);
Zsim = cumsum(xnorm2)*sqrt(prec);
Z2sim = cumsum(xnorm3)*sqrt(prec);
% observed process
Bobs = Bsim(index_obs==1);
Zobs = Zsim(index_obs==1);
Z2obs = Z2sim(index_obs==1);
% annual data
Bann = Bsim(index==1);
Zann = Zsim(index==1);
Z2ann = Z2sim(index==1);
% creates index of observed out of the simulated Brownian motion
Xsel_sim = (1:(nPeriods-1)/prec);
Xsel_obs = (1:(nPeriods-1)/freq)./prec*freq;
Xsel_ann = (1:(nPeriods-1))./prec;

% if figures == 1
%     figure(10);
%     subplot(3,1,1)
%     plot(0:nPeriods/prec,[Bsim_init Bsim])
%     hold on
%     title('simulated data');
%     plot(Xsel_ann,Bsim(Xsel_ann),'r.')
%     axis([0,nPeriods/prec,min([Bsim*.9 Bsim*1.1]),max([Bsim*.9 Bsim*1.1])])
%     hold off
% 
%     subplot(3,1,2)
%     plot(0:(nPeriods-1)/freq,[Bsim_init Bobs]);
%     hold on
%     title('observed data');
%     axis([0,(nPeriods-1)/freq,min([Bsim*.9 Bsim*1.1]),max([Bsim*.9 Bsim*1.1])])
%     hold off
% 
%     subplot(3,1,3)
%     plot(0:(nPeriods-1),[Bsim_init Bann]);
%     hold on
%     title('annual data');
%     axis([0,(nPeriods-1),min([Bsim*.9 Bsim*1.1]),max([Bsim*.9 Bsim*1.1])])
%     hold off
% end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1 Lucas' fruit tree economy (two assets)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (solution == 1) 
    Asim = Asim_init*exp((mu_bar-.5*sigma_bar^2).*(1:nPeriods/prec).*prec+(sigma_bar*(Bsim-Bsim_init))+(nu_bar*(xpois-Nsim_init)));
    y = log(Asim);
    waitbar(loop/loops)
    B_init(loop) = Bsim_init;
    Z_init(loop) = Zsim_init;
    N_init(loop) = Nsim_init;
    N2_init(loop) = N2sim_init;
end    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 3 neoclassical production economy (nonlinear solution)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (solution == 3) 
    % parameters of the geometric jump-diffusion process
    c1 = (1-alpha)/(gamma*alpha);
    c2 = alpha*gamma/(1-alpha)*mu_bar+alpha*gamma*delta-.5*alpha*gamma*(alpha-2)*sigma^2;
    % initial values 
    rsim_init = c2;
    L  = 1;
    % initial values 
    rsim(1:1/prec)= rsim_init; 
    Asim(1:1/prec)= Asim_init; 
    rds(1:1/prec) = prec*rsim_init; 
    Theta(1:1/prec) = 1;
    Bsim_init = Bsim(1/prec);
    Zsim_init = Zsim(1/prec);
    Nsim_init = xpois(1/prec);
    N2sim_init = xpois2(1/prec);
    
    % computing realized values of the underlying processes with precise timing
    for i = 1/prec+1 : max(nPeriods/prec)
        Asim(i) = Asim_init*exp((mu_bar-.5*sigma_bar^2).*(i*prec-1)+sigma_bar*(Bsim(i)-Bsim_init)+nu_bar*(xpois(i)-Nsim_init));
        Theta(i) = exp((c1*c2-1/2*(alpha-1)^2*sigma^2-1/2*sigma_bar^2)*(i*prec-1)+sigma*(alpha-1)*(Zsim(i)-Zsim_init)+sigma_bar*(Bsim(i)-Bsim_init)+nu*(alpha-1)*(xpois2(i)-N2sim_init)+nu_bar*(xpois(i)-Nsim_init));
        rsim(i) = Theta(i)*(rsim_init^(-1)+c1*traprule(Theta(1/prec:i),(1/prec:i)*prec))^(-1); 
        rds(i)= traprule(rsim(i-1:i),(1:2)*prec); %  trapz(rsim(i-1:i))*prec
    end
    y=log(Asim(1/prec+1:end));
    k=log((rsim(1/prec+1:end)./(alpha*Asim(1/prec+1:end)*L^(1-alpha))).^(1/(alpha-1)));
    Ksim_init = (rsim_init./(alpha*Asim_init*L^(1-alpha))).^(1/(alpha-1));
    rds =rds(1/prec+1:end);
    rsim =rsim(1/prec+1:end);
    B_init(loop) = Bsim_init;
    Z_init(loop) = Zsim_init;
    N_init(loop) = Nsim_init;
    N2_init(loop) = N2sim_init;

    waitbar(loop/loops)
end    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Write sample paths to file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (solution == 1) 
    outr_y=[outr_y y'];
end
if (solution == 3) 
    outr_y=[outr_y y'];
    outr_k=[outr_k k'];
    outr_rds=[outr_rds rds'];
    outr_rsim=[outr_rsim rsim'];
end

% Auxilary functions
function traprule=traprule(f,t)
T=zeros(size(t));
for i=2:length(t)
    T(i)=(t(i)-t(i-1))/2*(f(i-1)+f(i));
end
traprule=sum(T);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% End of file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
